#!/usr/bin/env perl
use strict;
use warnings;
use 5.10.0;

use lib '/usr/share/longview/lib';

use JSON::PP;
use Sys::Hostname;
use LWP::UserAgent;
use Compress::Zlib;
use Linode::Longview::DataGetter qw(get);
use Linode::Longview::Util qw(constantPush flattenData slurp_file daemonize_self check_already_running);

if ($< != 0) {
	print "Longview must be run as root in order to collect data\n";
	exit 1;
}

if (my $pid = check_already_running()){
	print "The Longview agent is already running as pid: $pid\n";
	exit 1;
}

my $ua    = LWP::UserAgent->new(
	timeout => 10,
	agent   => "Linode Longview 1.0"
);
my $url = 'http://longview1.linode.com/cgi.pl';

my $stats;
$stats->{apikey}   = slurp_file('/etc/longview.key');
unless ($stats->{apikey}){
	print "No API Key :(\n";
	exit 1;
}
$stats->{version}  = '1.0';
$stats->{payload}  = [];

my $data={};

my $debug = 0;
daemonize_self() unless $debug;

my $quit = 0;
$SIG{TERM} = $SIG{INT} = $SIG{HUP} = $SIG{QUIT} = sub { $quit = 1 };
my $counter = 0;
while (!$quit) {
	$data->{timestamp} = time;
	get($_,$data) foreach (qw(Processes Uptime Ports Load  Memory CPU Network Disk));
	unless ($counter % 1440) {
		get("SysInfo",$data);
		get('Packages', $data);
	}

	constantPush($stats->{payload},$data);
	$data = {};

	my $req = $ua->post(
		$url,
		Content_Type => 'form-data',
		Content => [
			'data' => [
				undef, 
				'json.gz', 
				'Content-Type' => 'application/json',
				'Content-Encoding' => 'gzip',
				'Content' => Compress::Zlib::memGzip(encode_json($stats))
			]   
		]   
	);  

	if ($req->is_success){
		@{$stats->{payload}} = () 
	}else{
		print $req->status_line."\n";
		print $req->decoded_content."\n";
	}
	$counter++;
	sleep 60 unless $debug;
	$quit = 1 if $debug;
}
