package Linode::Longview::DataGetter::Ports;

use Exporter;
@ISA       = qw(Exporter);
@EXPORT_OK = qw(get);

use strict;
use warnings;

use Linode::Longview::Util qw(slurp_file $procfs);
use Linode::Longview::DataGetter::Processes qw(processInfo processList);

use Data::Printer;

sub get {
	my $dataref = shift;
	my @procs = processList();
	my ( %listening,    %active );
	my ( @listeningOut, @activeOut );

	for my $proc (@procs) {
		my %procData = processInfo($proc) or next;
		my @fds = glob "${procfs}/${proc}/fd/*";

		for my $fd (@fds) {
			my $rl = readlink $fd;

			# if the symlink went away, the process probably did too.
			next unless ($rl);
			next unless ($rl) = ( $rl =~ m/socket:\[(\d+)\]/ );

			my ( $socketType, $sip, $sport, $rip, $rport ) = getSocketInfo($rl);

			# not a network connection
			next unless $socketType;

			$sip   = translateIP($sip);
			$sport = hex $sport;

			# active connection
			if ( hex $rport ) {
				if ( exists( $active{ $procData{name} . $procData{user} } ) ) {
					$active{ $procData{name} . $procData{user} }{count}++;
				}
				else {
					$active{ $procData{name} . $procData{user} } = {
						name  => $procData{name},
						user  => $procData{user},
						count => 1,
					};
				}
			}
			else {
				# listening port
				$listening{ $procData{name}
						. $procData{user}
						. $socketType
						. $sip
						. $sport } = {
					name => $procData{name},
					user => $procData{user},
					type => $socketType,
					ip   => $sip,
					port => $sport + 0,
						}
					unless exists(
					$listening{
							  $procData{name}
							. $procData{user}
							. $socketType
							. $sip
							. $sport
					} );
			}
		}
	}
	push @listeningOut, $listening{$_} foreach keys(%listening);
	push @activeOut,    $active{$_}    foreach keys(%active);
	$dataref->{INSTANT}->{"Ports.listening"} = \@listeningOut;
	$dataref->{INSTANT}->{"Ports.active"} = \@activeOut;
	return $dataref;
}

sub getSocketInfo {
	my $inode = shift;
	foreach my $type (qw(tcp udp tcp6 udp6)) {
		my @socks = slurp_file("/proc/net/$type");
		foreach my $sock (@socks) {
			if ( ( split /\s+/, $sock )[10] eq $inode ) {
				my ( $sip, $sport, $rip, $rport );
				$sock =~ m/\s+\S+\s+(.*?):(\S+)\s+(.*?):(\S+)\s+/;
				return ( $type, $1, $2, $3, $4 );
			}
		}
	}
}

sub translateIP {
	my $rawIP = shift;
	if ( length("$rawIP") > 8 ) {
		my @ip;
		push( @ip, reverse( unpack( "a2" x 4 ) ) )
			foreach ( unpack( "a8" x 4, "$rawIP" ) );
		my $v6 = join( ":", unpack( "a4" x 8, join( "", @ip ) ) );
		$v6 =~ s/(0000:?)+/:/;
		$v6 =~ s/:$/::/;
		return $v6;
	}
	$rawIP = hex $rawIP;
	return join( ".", reverse( unpack( 'C4', pack( 'N', $rawIP ) ) ) );
}
