package Linode::Longview::DataGetter::SysInfo;

use Sys::Hostname;

use Exporter;
@ISA       = qw(Exporter);
@EXPORT_OK = qw(get);

use Linode::Longview::Util qw(slurp_file detect_system $procfs $sysfs $arch);
use Linux::Distribution qw(distribution_name distribution_version);
use POSIX qw(uname);

sub get {
	my $dataref = shift;

	$dataref->{SYSINFO}->{"SysInfo.os.dist"}			= 'unknown';
	$dataref->{SYSINFO}->{"SysInfo.os.distversion"} 	= 'unknown';
	if ( my $distro = distribution_name() ) {
		$dataref->{SYSINFO}->{"SysInfo.os.dist"} 		= ucfirst($distro);
		$dataref->{SYSINFO}->{"SysInfo.os.distversion"} = distribution_version();
	}

	my ( $sysname, undef, $release, $version, $machine ) = uname();
	$dataref->{SYSINFO}->{"SysInfo.kernel"} = "$sysname $release";

	my @cpu_info = slurp_file( $procfs . 'cpuinfo' )
		or die "Couldn't read ${procfs}cpuinfo: $!";

	$dataref->{SYSINFO}->{"SysInfo.cpu.cores"} = grep {/^processor/} @cpu_info;
	#<<<
	($dataref->{SYSINFO}->{"SysInfo.cpu.type"} = (map { /^model name\s+:(?<model>.*)$/; $+{model} || ()} @cpu_info)[0]) =~ s/^\s+//;
	#>>>
	$dataref->{SYSINFO}->{"SysInfo.cpu.type"} =~ s/\s\s+/ /g;
	$dataref->{SYSINFO}->{"SysInfo.type"} = detect_system();
	$dataref->{SYSINFO}->{"SysInfo.arch"} = $arch;
	$dataref->{SYSINFO}->{"SysInfo.hostname"} = hostname;

	return $dataref;
}

1;
